@@ -1,9 +1,9 @@
 // SPDX-FileCopyrightText: 2016 Sandro Knauß <knauss@kolabsystems.com>
 // SPDX-License-Identifier: LGPL-2.0-or-later
 
+#include "partmodel.h"
 #include <MimeTreeParserCore/ObjectTreeParser>
 
-#include <QDebug>
 #include <QTest>
 #include <QTimeZone>
 
@@ -361,18 +361,11 @@ private Q_SLOTS:
 
         QVERIFY(otp.plainTextContent().contains(QString::fromUtf8("ohno öäü")));
 
-        auto signaturePart = part->signatures().first();
-        QCOMPARE(signaturePart->partMetaData()->isGoodSignature, true);
-        QCOMPARE(signaturePart->partMetaData()->isTrusted(), true);
-        QCOMPARE(signaturePart->partMetaData()->sigSummary & GpgME::Signature::KeyMissing, false);
-        QCOMPARE(signaturePart->partMetaData()->sigSummary & GpgME::Signature::KeyExpired, false);
-        QCOMPARE(signaturePart->partMetaData()->sigSummary & GpgME::Signature::KeyRevoked, false);
-        QCOMPARE(signaturePart->partMetaData()->sigSummary & GpgME::Signature::SigExpired, false);
-        QCOMPARE(signaturePart->partMetaData()->sigSummary & GpgME::Signature::CrlMissing, false);
-        QCOMPARE(signaturePart->partMetaData()->sigSummary & GpgME::Signature::CrlTooOld, false);
-        QCOMPARE(signaturePart->partMetaData()->keyId, QByteArray{"8D9860C58F246DE6"});
-        QCOMPARE(signaturePart->partMetaData()->signer, QLatin1StringView{"unittest key (no password) <test@kolab.org>"});
-        QCOMPARE(signaturePart->partMetaData()->signerMailAddresses, QStringList{{QStringLiteral("test@kolab.org")}});
+        const auto details = PartModel::signatureDetails(part.get());
+        QCOMPARE(details,
+                 QStringLiteral("Signature created on Tuesday, August 25, 2015 2:47:11\u202FPM UTC with certificate: <a "
+                                "href=\"key:1BA323932B3FAA826132C79E8D9860C58F246DE6\">unittest key (no password) &lt;test@kolab.org&gt; "
+                                "(8D98 60C5 8F24 6DE6)</a><br/>The signature is valid and the certificate's validity is ultimately trusted."));
     }
 
     void testEncryptedAndSigned()
@@ -389,9 +382,11 @@ private Q_SLOTS:
         QCOMPARE(part->signatureState(), MimeTreeParser::KMMsgFullySigned);
         QVERIFY(otp.plainTextContent().contains(QString::fromUtf8("encrypted message text")));
 
-        auto signaturePart = part->signatures().first();
-        QCOMPARE(signaturePart->partMetaData()->keyId, QByteArray{"8D9860C58F246DE6"});
-        QCOMPARE(signaturePart->partMetaData()->isGoodSignature, true);
+        const auto details = PartModel::signatureDetails(part.get());
+        QCOMPARE(details,
+                 QStringLiteral("Signature created on Tuesday, October 6, 2015 10:11:52\u202FAM UTC with certificate: <a "
+                                "href=\"key:1BA323932B3FAA826132C79E8D9860C58F246DE6\">unittest key (no password) &lt;test@kolab.org&gt; "
+                                "(8D98 60C5 8F24 6DE6)</a><br/>The signature is valid and the certificate's validity is ultimately trusted."));
     }
 
     void testOpenpgpMultipartEmbedded()
@@ -423,11 +418,11 @@ private Q_SLOTS:
         QCOMPARE(part->signatureState(), MimeTreeParser::KMMsgFullySigned);
         QCOMPARE(otp.plainTextContent(), QString::fromUtf8("test\n\n-- \nThis is a HTML signature.\n"));
 
-        auto signaturePart = part->signatures().first();
-        QVERIFY(signaturePart->partMetaData()->keyId.endsWith(QByteArray{"2E3B7787B1B75920"}));
-        // We lack the public key for this message
-        QCOMPARE(false, signaturePart->partMetaData()->isGoodSignature);
-        QCOMPARE(GpgME::Signature::KeyMissing, signaturePart->partMetaData()->sigSummary);
+        const auto details = PartModel::signatureDetails(part.get());
+        QCOMPARE(details,
+                 QStringLiteral("Signature created on Tuesday, April 24, 2018 4:47:20\u202FPM UTC using an unknown certificate "
+                                "with fingerprint <br/>CBD1 1648 5DB9 560C A3CD  91E0 2E3B 7787 B1B7 5920<br/>You can search "
+                                "the certificate on a keyserver or import it from a file."));
     }
 
     void testAppleHtmlWithAttachments()
@@ -653,7 +648,7 @@ private Q_SLOTS:
         QCOMPARE(part->charset(), QStringLiteral("utf-8").toLocal8Bit());
         QCOMPARE(part->encryptions().size(), 1);
         QCOMPARE(part->signatures().size(), 1);
-        QCOMPARE(part->signatures()[0]->partMetaData()->isGoodSignature, true);
+        QCOMPARE(PartModel::signatureSecurityLevel(part.get()), PartModel::Good);
         QCOMPARE(part->encryptionState(), MimeTreeParser::KMMsgFullyEncrypted);
         QCOMPARE(part->signatureState(), MimeTreeParser::KMMsgFullySigned);
         auto contentAttachmentList = otp.collectAttachmentParts();
@@ -683,7 +678,7 @@ private Q_SLOTS:
         QCOMPARE(part->text(), QString());
         QCOMPARE(part->charset(), QStringLiteral("UTF-8").toLocal8Bit());
         QCOMPARE(part->signatures().size(), 1);
-        QCOMPARE(part->signatures()[0]->partMetaData()->isGoodSignature, true);
+        QCOMPARE(PartModel::signatureSecurityLevel(part.get()), PartModel::Good);
         auto contentAttachmentList = otp.collectAttachmentParts();
         QCOMPARE(contentAttachmentList.size(), 1);
     }
