@@ -3,17 +3,49 @@
 
 #include "partmodel.h"
 
+#include "enums.h"
 #include "htmlutils.h"
+#include "messagepart.h"
+#include "mimetreeparser_core_debug.h"
 #include "objecttreeparser.h"
 #include "utils.h"
 
 #include <KLocalizedString>
+#include <Libkleo/Compliance>
+#include <Libkleo/Formatting>
+#include <Libkleo/KeyCache>
 
 #include <QDebug>
 #include <QGpgME/Protocol>
 #include <QRegularExpression>
 #include <QStringLiteral>
 #include <QTextDocument>
+#include <verificationresult.h>
+
+static std::optional<GpgME::Signature> signatureFromMessagePart(MimeTreeParser::MessagePart *messagePart)
+{
+    const auto signatureState = messagePart->signatureState();
+    const bool messageIsSigned = signatureState == MimeTreeParser::KMMsgPartiallySigned || signatureState == MimeTreeParser::KMMsgFullySigned;
+
+    if (!messageIsSigned) {
+        return std::nullopt;
+    }
+
+    const auto signatureParts = messagePart->signatures();
+    Q_ASSERT(!signatureParts.isEmpty());
+    if (signatureParts.empty()) {
+        return std::nullopt;
+    }
+    const auto signaturePart = signatureParts.front(); // TODO add support for multiple signature
+
+    const auto signatures = signaturePart->partMetaData()->verificationResult.signatures();
+    Q_ASSERT(!signatures.empty());
+    if (signatures.empty()) {
+        return std::nullopt;
+    }
+    const auto signature = signatures.front(); // TODO add support for multiple signature
+    return signature;
+}
 
 // We return a pair containing the trimmed string, as well as a boolean indicating whether the string was trimmed or not
 std::pair<QString, bool> PartModel::trim(const QString &text)
@@ -178,7 +210,7 @@ public:
         for (const auto &part : parts) {
             if (part->node()) {
                 const auto contentType = part->node()->contentType();
-                if (contentType && contentType->hasParameter(QStringLiteral("protected-headers"))) {
+                if (contentType && contentType->hasParameter("protected-headers")) {
                     const auto contentDisposition = part->node()->contentDisposition();
                     if (contentDisposition && contentDisposition->disposition() == KMime::Headers::CDinline) {
                         continue;
@@ -223,9 +255,7 @@ PartModel::PartModel(std::shared_ptr<MimeTreeParser::ObjectTreeParser> parser)
 {
 }
 
-PartModel::~PartModel()
-{
-}
+PartModel::~PartModel() = default;
 
 void PartModel::setShowHtml(bool html)
 {
@@ -271,23 +301,24 @@ bool PartModel::containsHtml() const
 
 QHash<int, QByteArray> PartModel::roleNames() const
 {
-    QHash<int, QByteArray> roles;
-    roles[TypeRole] = QByteArrayLiteral("type");
-    roles[ContentRole] = QByteArrayLiteral("content");
-    roles[IsEmbeddedRole] = QByteArrayLiteral("embedded");
-    roles[IsEncryptedRole] = QByteArrayLiteral("encrypted");
-    roles[IsSignedRole] = QByteArrayLiteral("signed");
-    roles[SecurityLevelRole] = QByteArrayLiteral("securityLevel");
-    roles[EncryptionSecurityLevelRole] = QByteArrayLiteral("encryptionSecurityLevel");
-    roles[SignatureSecurityLevelRole] = QByteArrayLiteral("signatureSecurityLevel");
-    roles[ErrorType] = QByteArrayLiteral("errorType");
-    roles[ErrorString] = QByteArrayLiteral("errorString");
-    roles[IsErrorRole] = QByteArrayLiteral("error");
-    roles[SenderRole] = QByteArrayLiteral("sender");
-    roles[SignatureDetails] = QByteArrayLiteral("signatureDetails");
-    roles[EncryptionDetails] = QByteArrayLiteral("encryptionDetails");
-    roles[DateRole] = QByteArrayLiteral("date");
-    return roles;
+    return {
+        {TypeRole, QByteArrayLiteral("type")},
+        {ContentRole, QByteArrayLiteral("content")},
+        {IsEmbeddedRole, QByteArrayLiteral("isEmbedded")},
+        {SidebarSecurityLevelRole, QByteArrayLiteral("sidebarSecurityLevel")},
+        {EncryptionSecurityLevelRole, QByteArrayLiteral("encryptionSecurityLevel")},
+        {SignatureSecurityLevelRole, QByteArrayLiteral("signatureSecurityLevel")},
+        {EncryptionSecurityLevelRole, QByteArrayLiteral("encryptionSecurityLevel")},
+        {ErrorType, QByteArrayLiteral("errorType")},
+        {ErrorString, QByteArrayLiteral("errorString")},
+        {IsErrorRole, QByteArrayLiteral("error")},
+        {SenderRole, QByteArrayLiteral("sender")},
+        {SignatureDetailsRole, QByteArrayLiteral("signatureDetails")},
+        {SignatureIconNameRole, QByteArrayLiteral("signatureIconName")},
+        {EncryptionDetails, QByteArrayLiteral("encryptionDetails")},
+        {EncryptionIconNameRole, QByteArrayLiteral("encryptionIconName")},
+        {DateRole, QByteArrayLiteral("date")},
+    };
 }
 
 QModelIndex PartModel::index(int row, int column, const QModelIndex &parent) const
@@ -327,30 +358,57 @@ SignatureInfo encryptionInfo(MimeTreeParser::MessagePart *messagePart)
     }
     return signatureInfo;
 };
-
-SignatureInfo signatureInfo(MimeTreeParser::MessagePart *messagePart)
+template<typename T>
+const T *findHeader(KMime::Content *content)
 {
-    SignatureInfo signatureInfo;
-    const auto signatures = messagePart->signatures();
-    if (signatures.size() > 1) {
-        qWarning() << "Can't deal with more than one signature";
+    if (!content) {
+        return {};
     }
-    for (const auto &signaturePart : signatures) {
-        signatureInfo.keyId = signaturePart->partMetaData()->keyId;
-        signatureInfo.cryptoProto = signaturePart->cryptoProto();
-        signatureInfo.keyMissing = signaturePart->partMetaData()->sigSummary & GpgME::Signature::KeyMissing;
-        signatureInfo.keyExpired = signaturePart->partMetaData()->sigSummary & GpgME::Signature::KeyExpired;
-        signatureInfo.keyRevoked = signaturePart->partMetaData()->sigSummary & GpgME::Signature::KeyRevoked;
-        signatureInfo.sigExpired = signaturePart->partMetaData()->sigSummary & GpgME::Signature::SigExpired;
-        signatureInfo.crlMissing = signaturePart->partMetaData()->sigSummary & GpgME::Signature::CrlMissing;
-        signatureInfo.crlTooOld = signaturePart->partMetaData()->sigSummary & GpgME::Signature::CrlTooOld;
-        signatureInfo.signer = signaturePart->partMetaData()->signer;
-        signatureInfo.isCompliant = signaturePart->partMetaData()->isCompliant;
-        signatureInfo.signerMailAddresses = signaturePart->partMetaData()->signerMailAddresses;
-        signatureInfo.signatureIsGood = signaturePart->partMetaData()->isGoodSignature;
-        signatureInfo.keyTrust = signaturePart->partMetaData()->keyTrust;
+    auto header = content->header<T>();
+    if (header || !content->parent()) {
+        return header;
     }
-    return signatureInfo;
+    return findHeader<T>(content->parent());
+}
+
+PartModel::SecurityLevel PartModel::signatureSecurityLevel(MimeTreeParser::MessagePart *messagePart)
+{
+    auto signature = signatureFromMessagePart(messagePart);
+    if (!signature) {
+        return SecurityLevel::Unknow;
+    }
+
+    const auto summary = signature->summary();
+
+    if (summary & GpgME::Signature::Summary::Red) {
+        return SecurityLevel::Bad;
+    }
+    if (summary & GpgME::Signature::Summary::Valid) {
+        return SecurityLevel::Good;
+    }
+
+    return SecurityLevel::NotSoGood;
+}
+
+QString PartModel::signatureDetails(MimeTreeParser::MessagePart *messagePart)
+{
+    auto signature = signatureFromMessagePart(messagePart);
+    if (!signature) {
+        return QString{};
+    }
+
+    // guess sender from mime node or parent node
+    auto from = findHeader<KMime::Headers::From>(messagePart->node());
+    if (from) {
+        const auto mailboxes = from->mailboxes();
+        if (!mailboxes.isEmpty()) {
+            auto mailBox = mailboxes.front();
+            if (mailBox.hasAddress()) {
+                return Kleo::Formatting::prettySignature(*signature, QString::fromUtf8(mailboxes.front().address()));
+            }
+        }
+    }
+    return Kleo::Formatting::prettySignature(*signature, {});
 }
 
 QVariant PartModel::data(const QModelIndex &index, int role) const
@@ -442,63 +500,65 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
             return messagePart->error();
         case ContentRole:
             return d->contentForPart(messagePart);
-        case IsEncryptedRole:
-            return messagePart->encryptionState() != MimeTreeParser::KMMsgNotEncrypted;
-        case IsSignedRole:
-            return messagePart->signatureState() != MimeTreeParser::KMMsgNotSigned;
-        case SecurityLevelRole: {
-            auto signature = messagePart->signatureState();
-            auto encryption = messagePart->encryptionState();
-            bool messageIsSigned = signature == MimeTreeParser::KMMsgPartiallySigned || signature == MimeTreeParser::KMMsgFullySigned;
-            bool messageIsEncrypted = encryption == MimeTreeParser::KMMsgPartiallyEncrypted || encryption == MimeTreeParser::KMMsgFullyEncrypted;
+        case SidebarSecurityLevelRole: {
+            const auto signature = index.data(SignatureSecurityLevelRole).value<SecurityLevel>();
+            const auto encryption = index.data(EncryptionSecurityLevelRole).value<SecurityLevel>();
 
-            if (messageIsSigned) {
-                const auto sigInfo = signatureInfo(messagePart);
-                if (!sigInfo.signatureIsGood) {
-                    if (sigInfo.keyMissing || sigInfo.keyExpired) {
-                        return SecurityLevel::NotSoGood;
-                    }
-                    return SecurityLevel::Bad;
-                }
+            if (signature == SecurityLevel::Bad || encryption == SecurityLevel::Bad) {
+                return SecurityLevel::Bad;
             }
-            // All good
-            if ((messageIsSigned || messageIsEncrypted) && !messagePart->error()) {
+
+            if (signature == SecurityLevel::NotSoGood || encryption == SecurityLevel::NotSoGood) {
+                return SecurityLevel::NotSoGood;
+            }
+
+            if (signature == SecurityLevel::Good || encryption == SecurityLevel::Good) {
                 return SecurityLevel::Good;
             }
-            // No info
+
             return SecurityLevel::Unknow;
         }
+        case SignatureSecurityLevelRole:
+            // Color displayed for the signature info box
+            return signatureSecurityLevel(messagePart);
         case EncryptionSecurityLevelRole: {
-            auto encryption = messagePart->encryptionState();
-            bool messageIsEncrypted = encryption == MimeTreeParser::KMMsgPartiallyEncrypted || encryption == MimeTreeParser::KMMsgFullyEncrypted;
+            // Color displayed for the encryption info box
+            const auto encryption = messagePart->encryptionState();
+            const bool messageIsEncrypted = encryption == MimeTreeParser::KMMsgPartiallyEncrypted || encryption == MimeTreeParser::KMMsgFullyEncrypted;
+
             if (messagePart->error()) {
                 return SecurityLevel::Bad;
             }
-            // All good
-            if (messageIsEncrypted) {
-                return SecurityLevel::Good;
-            }
-            // No info
-            return SecurityLevel::Unknow;
+
+            return messageIsEncrypted ? SecurityLevel::Good : SecurityLevel::Unknow;
         }
-        case SignatureSecurityLevelRole: {
-            auto signature = messagePart->signatureState();
-            bool messageIsSigned = signature == MimeTreeParser::KMMsgPartiallySigned || signature == MimeTreeParser::KMMsgFullySigned;
-            if (messageIsSigned) {
-                const auto sigInfo = signatureInfo(messagePart);
-                if (!sigInfo.signatureIsGood) {
-                    if (sigInfo.keyMissing || sigInfo.keyExpired) {
-                        return SecurityLevel::NotSoGood;
-                    }
-                    return SecurityLevel::Bad;
-                }
-                return SecurityLevel::Good;
+        case EncryptionIconNameRole: {
+            const auto encryption = messagePart->encryptionState();
+            const bool messageIsEncrypted = encryption == MimeTreeParser::KMMsgPartiallyEncrypted || encryption == MimeTreeParser::KMMsgFullyEncrypted;
+
+            if (messagePart->error()) {
+                return QStringLiteral("data-error");
             }
-            // No info
-            return SecurityLevel::Unknow;
+
+            return messageIsEncrypted ? QStringLiteral("mail-encrypted") : QString();
         }
-        case SignatureDetails:
-            return QVariant::fromValue(signatureInfo(messagePart));
+        case SignatureIconNameRole: {
+            auto signature = signatureFromMessagePart(messagePart);
+            if (!signature) {
+                return QString{};
+            }
+
+            const auto summary = signature->summary();
+            if (summary & GpgME::Signature::Valid) {
+                return QStringLiteral("mail-signed");
+            } else if (summary & GpgME::Signature::Red) {
+                return QStringLiteral("data-error");
+            } else {
+                return QStringLiteral("data-warning");
+            }
+        }
+        case SignatureDetailsRole:
+            return signatureDetails(messagePart);
         case EncryptionDetails:
             return QVariant::fromValue(encryptionInfo(messagePart));
         case ErrorType:
@@ -510,11 +570,9 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
                     if (encryptedMessagePart->isNoSecKey()) {
                         QString errorMessage;
                         if (encryptedMessagePart->cryptoProto() == QGpgME::smime()) {
-                            errorMessage +=
-                                i18ndc("mimetreeparser", "@info:status", "You cannot decrypt this message.");
+                            errorMessage += i18ndc("mimetreeparser", "@info:status", "You cannot decrypt this message.");
                         } else {
-                            errorMessage +=
-                                i18ndc("mimetreeparser", "@info:status", "You cannot decrypt this message.");
+                            errorMessage += i18ndc("mimetreeparser", "@info:status", "You cannot decrypt this message.");
                         }
                         if (!encryptedMessagePart->decryptRecipients().empty()) {
                             errorMessage += QLatin1Char(' ')
